#!/usr/bin/env python3

# Copyright (C) 2013  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

'''
This works but Pandoc does not respect table alignment.
'''

from Xac.common import *
from Tabulator import Tabulator

def get_xac_markdown_dir_table(
  xac, dirpath, dirnames, filenames,
  fmt_mtime=format_iso_display_time,
  include_size = True,
  include_mtime = True,
  impl = None):
  """Get the Xac XHTML directory table for the given path and files.

  fmt_mtime:
    A function that accepts the file mtime and returns a formatted string.
  """

  dirnames = sorted(dirnames, key=str.lower)
  filenames = sorted(filenames, key=str.lower)
  filenames = zip(filenames, xac.map_markdown_filenames(dirpath,filenames,same_length=True))

  headers = ['File']
  rows = [headers]
  align = ('l', 'r')

  if include_size:
    headers.append('Size')
  if include_mtime:
    headers.append('Last Modified')

  for x,typ in \
    [(x, 'dir') for x in dirnames] + \
    [(x, 'file') for x in filenames]:

    if typ == 'file':
      y = x[1]
      x = x[0]
    else:
      y = x

    if x[0] == '.':
      continue

    path = os.path.join(dirpath, x)
    row = ['[%s](%s)' % (x,y)]
    if include_size or include_mtime:
      st = os.stat(path)

    if include_size:
      row.append(format_size(st.st_size))

    if include_mtime:
      row.append(fmt_mtime(st.st_mtime))
    rows.append(row)

  tab = Tabulator(rows, cols=align, headers=align)
  return tab.to_markdown_table()
