#!/usr/bin/env python3

# Copyright (C) 2012  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from Xac.common import *
from Xac.xml import html_svg_object

from xml.dom.minidom import parseString
import openbabel as ob

def inchi_to_inchi_key(conv, mol, inchi):
  # TODO
  # Remove this once the inchi->inchikey bug in openbabel has been fixed..
  conv.SetInAndOutFormats("inchi", "smi")
  conv.ReadString(mol, inchi)
  smi = conv.WriteString(mol)
  conv.SetInAndOutFormats("smi", "inchikey")
  conv.ReadString(mol, smi)
  return conv.WriteString(mol).strip()



def fix_babel_svg(svg):
  """This removes the PITA viewbox created by openbabel's SVG output, and
  adjusts the image to better fit the data. The results are similar to the
  output of the babel command-line tool."""
  xml = parseString(svg)
  # Remove background rectangle.
  svg = xml.documentElement

  for g in xml.getElementsByTagName('g'):
    inner_svg = g.getElementsByTagName('svg')[0]
    viewbox = inner_svg.getAttribute('viewBox')
    # Intentionally inverted to ensure min and max conditions below create a
    # tight box.
    xmax, ymax, xmin, ymin = (float(x) for x in viewbox.split(' '))

    for line in g.getElementsByTagName('line'):
      x1 = float(line.getAttribute('x1'))
      x2 = float(line.getAttribute('x2'))
      y1 = float(line.getAttribute('y1'))
      y2 = float(line.getAttribute('y2'))
      xmin = min(xmin, x1, x2)
      xmax = max(xmax, x1, x2)
      ymin = min(ymin, y1, y2)
      ymax = max(ymax, y1, y2)
      line.setAttribute('stroke-width', '1')

    for text in g.getElementsByTagName('text'):
      x = float(text.getAttribute('x'))
      y = float(text.getAttribute('y'))
      fs = float(text.getAttribute('font-size'))
      l = len(text.firstChild)
      dx = l * fs
      dy = fs
      xmin = min(xmin, x-dx)
      xmax = max(xmax, x+dx)
      ymin = min(ymin, y-dy)
      ymax = max(ymax, y+dy)
      line.setAttribute('stroke-width', '1')

    # Shift everything for a tight fit.
    for line in g.getElementsByTagName('line'):
      for attr, dx in (
        ('x1', xmin),
        ('x2', xmin),
        ('y1', ymin),
        ('y2', ymin)
      ):
        x = float(line.getAttribute(attr))
        line.setAttribute(attr, '%f' % (x-dx+5))

    for text in g.getElementsByTagName('text'):
      for attr, dx in (
        ('x', xmin),
        ('y', ymin),
      ):
        x = float(text.getAttribute(attr))
        text.setAttribute(attr, '%f' % (x-dx+5))


#     svg.setAttribute('viewBox', '%f %f %f %f' % (xmin, ymin, xmax, ymax))
    svg.setAttribute('width', '%d' % (xmax-xmin+10))
    svg.setAttribute('height', '%d' % (ymax-ymin+10))
    svg.setAttribute('x', '5') #'%d' % xmin)
    svg.setAttribute('y', '5') #'%d' % ymin)
    for attr in ('id','viewBox'): #'height', 'width'):
      svg.removeAttribute(attr)
    for attr in ('stroke-linecap', 'stroke-width', 'font-family'):
      svg.setAttribute(attr, inner_svg.getAttribute(attr))
    for child in list(svg.childNodes):
      svg.removeChild(child)
    for child in list(inner_svg.childNodes):
      inner_svg.removeChild(child)
      try:
        child.tagName
        svg.appendChild(child)
      except AttributeError:
        pass
    break



  # minidom's unexpected text nodes are so annoying
  return '\n'.join([x.strip() for x in xml.toprettyxml(indent='  ').split('\n')]).replace('\n\n\n', '\n')

def get_default_options(outdir, urldir, cls=None):
  """Get default options given a real output directory, a relative directory
  on the server, and an optional temporary directory.

  An HTML class may also be include for the image.
  """
  options = {
    'args' : ['-d', '-xwC', '--genalias', '-xA'],
    'options' : 'dAwCN1',
    'outdir' : outdir,
    'urldir' : urldir
    }

  if cls:
    options['class'] = cls

  return options




def inchi_svg(text, srcpath, chunks, outtype, options):
  if outtype != 'html':
    raise TypeError("unsupported output type: %s", outtype)

  outdir = options['outdir']
  urldir = options['urldir']
  make_dirs(outdir, only_parents=False)

  try:
    cls = [options['class']]
  except KeyError:
    cls = []

  conv = ob.OBConversion()
  mol = ob.OBMol()
  for tag, data in chunks.items():
    data = data.strip()
    if data[:6] != 'InChI=':
      data = 'InChI=' + data
    inchi_key = inchi_to_inchi_key(conv, mol, data)
    imgpath = os.path.join(outdir, inchi_key + '.svg')
    if not os.path.exists(imgpath):
#       cmd = ['babel', '-i', 'inchi', '-o', 'svg', '-O', imgpath] + options['args']
#       pipe2(cmd, data)

      conv.SetOutFormat('svg')
      conv.SetOptions(options['options'], conv.OUTOPTIONS)
      svg = conv.WriteString(mol).strip()
      svg = fix_babel_svg(svg)
      save_file(imgpath, svg)

    replacement = html_svg_object(imgpath.replace(outdir, urldir, 1), cls)
    text = text.replace(tag, replacement)
  return text
