#!/usr/bin/env python3

# Copyright (C) 2012  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from Xac.common import *
from Xac.xml import html_svg_object

import re


def get_default_options(outdir, urldir, cls=None):
  """Get default options given a real output directory, a relative directory
  on the server, and an optional temporary directory.

  An HTML class may also be include for the image.
  """
  options = {
    'outdir' : outdir,
    'urldir' : urldir
    }

  if cls:
    options['class'] = cls

  return options



def graphviz_svg(text, srcpath, obj, outtype, options):
  """Insert a Graphviz graph.

  Use graph properties to control the output type (e.g. neato, circo).
  """
  outdir = options['outdir']
  urldir = options['urldir']
  make_dirs(outdir, only_parents=False)

  if outtype != 'html':
    raise ValueError("unrecognized output type: %s", outtype)

  try:
    cls = [options['class']]
  except KeyError:
    cls = []

  cmd = ['dot', '-T', 'svg', '-o']
  for tag, data in obj.items():
    data = data.strip()
    m = re.match(r'\s*(?:di)?graph\s+"(.+?)"', data)
    if m:
      filename = m.group(1).replace(' ','_') + '.svg'
    else:
      filename = hash_string(data, 'md5') + '.svg'
    imgpath = os.path.join(outdir, filename)
    url = os.path.join(urldir, filename)
    replacement = html_svg_object(url, cls)
#     if not os.path.exists(imgpath):
    make_dirs(imgpath)
    pipe2(cmd + [imgpath], data)
#     text = text.replace('<p>' + tag + '</p>', replacement)
    text = text.replace(tag, replacement)
  return text
