#!/usr/bin/env python3

# Copyright (C) 2013  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from Xac.common import *
from xml.sax.saxutils import escape
from os import readlink
from os.path import dirname, join, normpath

def resolve_path(fpath, srcpath):
  if fpath == '%SELF%':
    return srcpath

  if fpath[0] != '/':
    # The source path is a symlink in the prepub directory. Trace it back to the
    # files directory.
    target = readlink(srcpath)
    realsrcpath = join(dirname(srcpath), target)
    fpath = join(dirname(realsrcpath), fpath)
  return normpath(fpath)

default_options = {
  'class' : 'xac_code',
  'style' : 'white-space: pre;',
  'post_process' : None,
}

def insert_file(text, srcpath, chunks, outtype, options=default_options):
  """
  Insert the contents of a file as a code block.

  The first line of the data chunk should be the path to the file to insert.

  Use the options parameter to change the class and/or style of the inserted
  block. The "post_process" key may also be set to a function that will modify
  the file contents before insertion, for example to change relative paths or
  hide sensitive data.
  """

  if outtype == 'html':
    attr = ''
    for k in ('class', 'style'):
      try:
        attr += ' %s="%s"' % (k, options[k])
      except KeyError:
        pass
  else:
    raise TypeError("unsupported output type: %s", outtype)

  for tag, data in chunks.items():
    fpath = data.split('\n', 1)[0].strip()
    fpath = resolve_path(fpath, srcpath)
    if outtype == 'html':
      with open(fpath) as f:
        data = f.read()
      try:
        # Do not post_process here. If the supplied function throws a key error,
        # it should be caught by the user, not here.
        post_process = options['post_process']
      except KeyError:
        post_process = None
      if post_process:
        data = post_process(data)
      replacement = ('<div%s>' % attr) + escape(data) + '</div>'
    else:
      raise TypeError("unsupported output type: %s", outtype)
    text = text.replace('<p>' + tag + '</p>', replacement)
    text = text.replace(tag, replacement)
  return text
