#!/usr/bin/env python3

# Copyright (C) >=2012  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from Xac.common import *
from Xac.plugins.insert_file import resolve_path

default_options = {
  'args' : ['--css', 'code.css'],
  'class' : 'xac_code'
}

def source_highlight_core(text, srcpath, chunks, outtype, options=default_options, from_file=False):
  """
  This is the common core of the other functions.

  Do not use it directly. Use the wrappers below.
  """

  if outtype == 'html':
    cmd = ['source-highlight', '-f', 'xhtml', '--no-doc', '-o', 'STDOUT'] + options['args']
    try:
      cls = ' class="%s"' % options['class']
    except KeyError:
      cls = ''
  else:
    raise TypeError("unsupported output type: %s", outtype)

  for tag, data in chunks.items():
    i = data.find('\n')
    lang = data[:i]
    data = data[i+1:]

    args = ['-s', lang]
    if outtype == 'html':
      if from_file:
        # TODO
        # Reconsider this if people are insane enought to include spaces at the
        # beginning or end of file names.
        fpath = data.split('\n',1)[0].strip()
        fpath = resolve_path(fpath, srcpath)
        args += ['-i', fpath]
        data = None
      replacement = ('<div%s>' % cls) + pipe2(cmd + args, data) + '</div>'
    else:
      raise TypeError("unsupported output type: %s", outtype)
    text = text.replace('<p>' + tag + '</p>', replacement)
    text = text.replace(tag, replacement)
  return text

def source_highlight(text, srcpath, chunks, outtype, options=default_options):
  """
  Highlight code with GNU Source-highlight.

  The first line of the data chunk should be a language name in the output of

      source-highlight --lang-list

  The rest of the data chunk should be the code to highlight.
  """

  return source_highlight_core(text, srcpath, chunks, outtype, options=default_options, from_file=False)


def source_highlight_file(text, srcpath, chunks, outtype, options=default_options):
  """
  Highlight code from a file with GNU Source-highlight.

  The first line of the data chunk should be a language name in the output of

      source-highlight --lang-list

  The second line should be the path to the file with the code.
  """

  return source_highlight_core(text, srcpath, chunks, outtype, options=default_options, from_file=True)
