#!/usr/bin/env python3

# Copyright (C) 2012  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from Xac.common import *
import re



def get_default_options(outdir, urldir, tmpdir=None, cls=None):
  """Get default options given a real output directory, a relative directory
  on the server, and an optional temporary directory.

  The temporary directory is only required for persistent LaTeX auxiliary files.

  An HTML class may also be include for the image.
  """
  options = {
    'args' : ['-i', '-T', '-D', '100'],
    'outdir' : outdir,
    'urldir' : urldir
    }

  if tmpdir:
    options['tmpdir'] = tmpdir

  if cls:
    options['class'] = cls

  return options



def tex2png(text, srcpath, chunks, outtype, options):
  """Use tex2png to convert (La)TeX to PNG."""

  # TODO: Restructure when adding other output types.
  if outtype != 'html':
    raise TypeError("unsupported output type: %s", outtype)


  outdir = options['outdir']
  urldir = options['urldir']
  make_dirs(outdir, only_parents=False)

  cmd = ['tex2png'] + options['args'] + ['-d', outdir]

  try:
    cmd += ['-t', options['tmpdir']]
    make_dirs(options['tmpdir'], only_parents=False)
  except KeyError:
    pass

  try:
    cls = ' class="%s"' % options['class']
  except KeyError:
    cls = ''

  reg = re.compile(r'depth=(-?\d+) height=-?\d+\nfile=(.*?)\n')

  for tag, data in chunks.items():
    data = data.strip()
    output = pipe2(cmd, data)
    replacement = ''
    m = reg.match(output);
    if m:
      depth = -int(m.group(1))
      filepath = m.group(2)
      # Synchronize file's mtime with parent directory's to prevent
      # unnecessary republishing.
      touch(filepath, os.path.dirname(filepath))
      url = os.path.join(urldir, os.path.basename(filepath))
      replacement = '<img src="%s" alt="image generated by tex2png" style="vertical-align:%dpx;"%s/>' % (url, depth, cls)
    text = text.replace(tag, replacement)
  return text
