#!/usr/bin/env python3

# Copyright (C) 2012-2013  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from Xac.common import *
from xml.dom.minidom import getDOMImplementation, parse as miniparse, parseString as miniparseString

import os
import re
import time


# Use these so that the the XML backend can be changed without changing this
# modules interface.
def parse(f):
  return miniparse(s)

def parseString(s):
  return miniparseString(s)



def get_xac_xhtml_nav_panels(xac, path, child_dirnames, child_filenames,
                            sibling_dirnames, sibling_filenames,
                            rootname='~', classes=[], impl=None):
  """Get the Xac XHTML navigation tree for the given path."""
  name = os.path.basename(path)
  if not impl:
    impl = getDOMImplementation()

  child_dirnames = sorted(child_dirnames)
  child_filenames = sorted(xac.map_markdown_filenames(path, child_filenames))
  sibling_dirnames = sorted(sibling_dirnames)
  sibling_filenames = [x for x in sorted(xac.map_markdown_filenames(
                        os.path.dirname(path), sibling_filenames))]


  doc = impl.createDocument(None, 'div', None)
  panel = doc.documentElement
  panel.setAttribute('id', 'xac_cwd_nav_panel')
  if classes:
    panel.setAttribute('class', ' '.join(classes))


  path_components = path.split('/')

#   h1 = doc.createElement('h1')
#   h1.appendChild(doc.createTextNode('CWD'))
#   nav_tree.appendChild(h1)
  cwd = doc.createElement('h1')
  panel.appendChild(cwd)

  if len(path_components) > 1:
    #cwd.appendChild(doc.createTextNode('/'))
    root = doc.createElement('a')
    root.setAttribute('href', '/')
    root.appendChild(doc.createTextNode(rootname))
    cwd.appendChild(root)

    link = '/'
    for comp in path_components[1:-1]:
      link += comp + '/'
      cwd.appendChild(doc.createTextNode('/'))
      c = doc.createElement('a')
      c.setAttribute('href', link)
      c.appendChild(doc.createTextNode(comp))
      cwd.appendChild(c)

    cwd.appendChild(doc.createTextNode('/' + path_components[-1]))

  else:
    cwd.appendChild(doc.createTextNode(rootname + '/'))

  if child_dirnames or child_filenames:
    ul = doc.createElement('ul')
    panel.appendChild(ul)
    for xs, cls, sfx in (
      (child_dirnames, 'dir', '/'),
      (child_filenames, 'file', ''),
    ):
      if xs:
        for x in xs:
          if x[0] == '.':
            continue
          li = doc.createElement('li')
          ul.appendChild(li)
          a = doc.createElement('a')
          a.setAttribute('class', 'xac_' + cls + '_link')
          a.setAttribute('href', x + sfx)
          a.appendChild(doc.createTextNode(x))
          li.appendChild(a)

  cwd_xhtml = panel.toxml()
  parent_xhtml = ''

  if sibling_dirnames or sibling_filenames:
    parent_href = os.path.dirname(path)
    doc = impl.createDocument(None, 'div', None)
    panel = doc.documentElement
    panel.setAttribute('id', 'xac_parent_nav_panel')
    if classes:
      panel.setAttribute('class', ' '.join(classes))

    cwd = doc.createElement('h1')
    panel.appendChild(cwd)
    a = doc.createElement('a')
    a.setAttribute('href', parent_href)
    if len(path_components) > 2:
      a.appendChild(doc.createTextNode(path_components[-2]))
    else:
      a.appendChild(doc.createTextNode(rootname))
    cwd.appendChild(a)

    if sibling_dirnames or sibling_filenames:
      ul = doc.createElement('ul')
      panel.appendChild(ul)
      for xs, cls, sfx in (
        (sibling_dirnames, 'dir', '/'),
        (sibling_filenames, 'file', ''),
      ):
        if xs:
          for x in xs:
            if x[0] == '.':
              continue
            li = doc.createElement('li')
            ul.appendChild(li)
            a = doc.createElement('a')
            if x == name:
              href = '.'
              a.setAttribute('class', 'xac_recursive_link')
            else:
              href = os.path.join(parent_href, x + sfx)
              a.setAttribute('class', 'xac_' + cls + '_link')
            a.setAttribute('href', href)
            a.appendChild(doc.createTextNode(x))
            li.appendChild(a)
    parent_xhtml = panel.toxml()

  return cwd_xhtml, parent_xhtml








def get_xac_xhtml_dir_table(xac, dirpath, dirnames, filenames,
  fmt_mtime=format_iso_display_time,
  include_size = True,
  include_mtime = True,
  impl = None):
  """Get the Xac XHTML directory table for the given path and files.

  fmt_mtime:
    A function that accepts the file mtime and returns a formatted string.
  """
  if not impl:
    impl = getDOMImplementation()

  dirnames = sorted(dirnames, key=str.lower)
  filenames = sorted(filenames, key=str.lower)
  filenames = zip(filenames, xac.map_markdown_filenames(dirpath,filenames,same_length=True))

  doc = impl.createDocument(None, 'table', None)
  table = doc.documentElement
  table.setAttribute('class', 'xac_directory_table')

  thead = doc.createElement('thead')
  table.appendChild(thead)

  tbody = doc.createElement('tbody')
  table.appendChild(tbody)

#   caption = doc.createElement('caption')
#   table.appendChild(caption)
#   caption.appendChild(doc.createTextNode('Directory Listing'))

  tr = doc.createElement('tr')
  thead.appendChild(tr)
  th = doc.createElement('th')
  tr.appendChild(th)
  th.appendChild(doc.createTextNode('File'))
  if include_size:
    th = doc.createElement('th')
    th.setAttribute('align', 'right')
    tr.appendChild(th)
    th.appendChild(doc.createTextNode('Size'))
  if include_mtime:
    th = doc.createElement('th')
    th.setAttribute('align', 'right')
    tr.appendChild(th)
    th.appendChild(doc.createTextNode('Last Modified'))

  for x,typ in \
    [(x, 'dir') for x in dirnames] + \
    [(x, 'file') for x in filenames]:

    if typ == 'file':
      y = x[1]
      x = x[0]
    else:
      y = x

    if x[0] == '.':
      continue

    path = os.path.join(dirpath, x)
    tr = doc.createElement('tr')
    tbody.appendChild(tr)

    td = doc.createElement('td')
    tr.appendChild(td)
    a = doc.createElement('a')
    a.setAttribute('href', y)
    if typ == 'dir':
      a.setAttribute('class', 'xac_dir_link')
    else:
      a.setAttribute('class', 'xac_file_link')
    td.appendChild(a)
    a.appendChild(doc.createTextNode(y))

    if include_size or include_mtime:
      st = os.stat(path)

    if include_size:
      td = doc.createElement('td')
      tr.appendChild(td)
      td.setAttribute('align', 'right')
      td.appendChild(doc.createTextNode(format_size(st.st_size)))

    if include_mtime:
      td = doc.createElement('td')
      tr.appendChild(td)
      td.setAttribute('align', 'right')
      mtime = fmt_mtime(st.st_mtime)
      td.appendChild(doc.createTextNode(mtime))

  return table.toxml()







def get_xac_xhtml_nav_bar(links, attrs=None, impl=None):
  """Get the Xac XHTML top navigation bar.

  links:
    A list of 4-tuples containing hrefs, names, ids, and classes.
  """
  if not impl:
    impl = getDOMImplementation()

  doc = impl.createDocument(None, 'ul', None)
  ul = doc.documentElement
  try:
    for name, value in attrs.items():
      ul.setAttribute(name, value)
  except AttributeError:
    pass

  for href, name, attrs in links:
    li = doc.createElement('li')
    ul.appendChild(li)
    try:
      for n, v in attrs.items():
        li.setAttribute(n, v)
    except AttributeError:
      pass
    a = doc.createElement('a')
    li.appendChild(a)
    a.setAttribute('href', href)
    a.appendChild(doc.createTextNode(name))
  return ul.toxml()






def html_svg_object(
  srcpath,
  classes=[],
  errmsg = 'error: Your browser does not support SVG image objects.'
  ):
  """Return an SVG object."""
  # This should work but Webkit allows the SVG object to overflow. Dealing with
  # SVG dimensions dynamically by modifying the attributes of the image itself
  # is a PITA. Hopefully this will improve in the future. Until then, use the
  # img tag which is currently widely supported.

  # http://caniuse.com/svg-img

  # Actually, I'm not even sure why I bothered with the object tag to begin
  # with.
#   return '<object class="%s" type="image/svg+xml" data="%s">%s</object><a href="%s">object source</a>' % \
#     (' '.join(classes), srcpath, errmsg, srcpath)
  return '<a href="%s"><img class="%s" src="%s" alt="svg image"/></a>' % (srcpath, ' '.join(classes), srcpath)





def clean_xml(node):
  """Remove the annoying textnodes that xml.dom.minidom inserts."""
  empty_text_nodes = []
  for child in node.childNodes:
    if child.nodeType == node.TEXT_NODE and not child.nodeValue.strip():
      empty_text_nodes.append(child)
    else:
      clean_xml(child)
  for child in empty_text_nodes:
    node.removeChild(child)



def toxml(doc, pretty=True):
  """Convert XML to string. Includes some extra formatting to handle minidom's
  text nodes."""
  if pretty:
    return re.sub(
      r'(?<=>)\n\s*([^\s<].*?)\s*\n\s*',
      r'\1',
      doc.toprettyxml(indent='  ', encoding='utf-8').decode()
    )
  else:
    return doc.toxml(encoding='utf-8').decode()



class AtomItem():

  def __init__(self, doc, parent, iD, title, updated=None, pretty=True):

    self.doc = doc
    self.parent = parent

    if not updated:
      updated = time.time()

    self.id = iD
    self.title = title
    self.update = updated
    self.pretty = pretty

    self.pre_blocks = dict()

    updated_str = self.format_time(updated)

    for tag, text in (('id', iD), ('title', title), ('updated', updated_str)):
      self.add_element(tag, text=text)



  # Nasty hack to preserve HTML formatting.
  def __str__(self):
    s = toxml(self.doc, self.pretty)
    for k, v in self.pre_blocks.items():
      s = s.replace(k, v, 1)
    return s


  def save(self, path):
    """Save document to file."""
    with open(path, 'w') as f:
      #self.doc.writexml(f, indent='  ')
      f.write(self.__str__())


  def format_time(self, t):
    """Return the RFC 3339 formatted time."""
    return format_rfc3339_time(t)


  def append(self, e):
    """Append an element.

    This is overridden in the AtomFeed subclass to keep entry elements at the
    end.
    """
    self.parent.appendChild(e)



  def add_element(self, tag, attr=None, text=None, xml=None, node=None, pre=None):
    """Add a generic element to the feed."""
    e = self.doc.createElement(tag)
    self.append(e)
    if attr:
      for n,v in attr.items():
        e.setAttribute(n,v)
    if text:
      e.appendChild(self.doc.createTextNode(text))
    elif pre:
      h = hash_string(pre, 'sha256')
      self.pre_blocks[h] = pre
      e.appendChild(self.doc.createTextNode(h))
    else:
      if xml:
        node = parseString(xml.strip()).documentElement
      if node:
        clean_xml(node)
        e.appendChild(node)



  def add_person(self, tag, name, uri=None, email=None):
    """Add a generic element to the feed."""
    e = self.doc.createElement(tag)
    self.append(e)
    ee = self.doc.createElement('name')
    e.appendChild(ee)
    ee.appendChild(self.doc.createTextNode(name))
    if uri:
      ee = self.doc.createElement('uri')
      e.appendChild(ee)
      ee.appendChild(self.doc.createTextNode(uri))
    if email:
      ee = self.doc.createElement('email')
      e.appendChild(ee)
      ee.appendChild(self.doc.createTextNode(email))



  def add_xml(self, f):
    """Add XML elements froma file."""
    doc = parse(f)
    self.append(doc.documentElement)


  def set_attribute(self, attr, val):
    """Set an attribute of this item."""
    self.parent.setAttribute(attr, val)





class AtomFeed(AtomItem):

  def __init__(self, iD, title, updated=None, impl=None, pretty=True):
    if not impl:
      self.impl = getDOMImplementation()
    else:
      self.impl = impl

    self.doc = self.impl.createDocument(None, 'feed', None)
    self.feed = self.doc.documentElement
    self.feed.setAttribute('xmlns', 'http://www.w3.org/2005/Atom')
    self.entry_items = list()

    super(self.__class__, self).__init__(
      self.doc, self.feed, iD, title, updated, pretty
    )

  # Nasty hack to preserve HTML formatting.
  def __str__(self):
    s = toxml(self.doc, self.pretty)
    for e in self.entry_items:
      for k, v in e.pre_blocks.items():
        s = s.replace(k, v, 1)
    return s



  def append(self, e):
    if e.tagName == 'entry':
      self.feed.appendChild(e)
    else:
      for child in self.feed.childNodes:
        if child.tagName == 'entry':
          self.feed.insertBefore(e, child)
          break
      else:
        self.feed.appendChild(e)



  def create_entry(self, iD, title, updated=None):
    """Create a new entry and return it for modification."""
    e = self.doc.createElement('entry')
    self.append(e)
    ei = AtomItem(self.doc, e, iD, title, updated, self.pretty)
    self.entry_items.append(ei)
    return ei



def create_xml_site_map(xac, dest_path, srv_url, impl=None):
  """Generate an XML site map of xac.pub_dir and save it to dest_path."""
  if not impl:
    impl = getDOMImplementation()
  doc = impl.createDocument(None, 'urlset', None)
  urlset = doc.documentElement
  urlset.setAttribute('xmlns', 'http://www.sitemaps.org/schemas/sitemap/0.9')
  for dirpath, dnames, fnames in os.walk(xac.pub_dir, followlinks=True):
    for name in dnames + fnames:
      if name[0] == '.':
        try:
          dnames.remove(name)
        except ValueError:
          pass
        continue
      path = os.path.join(dirpath, name)
      url = path.replace(xac.pub_dir, srv_url, 1)
#       prepub_path = path.replace(xac.pub_dir, xac.prepub_dir, 1)
      urle = doc.createElement('url')
      urlset.appendChild(urle)

      loc = doc.createElement('loc')
      urle.appendChild(loc)
      loc.appendChild(doc.createTextNode(url))

      lastmod = doc.createElement('lastmod')
      urle.appendChild(lastmod)
      mtime = format_rfc3339_time(os.path.getmtime(path))
      lastmod.appendChild(doc.createTextNode(mtime))

#       markdown_path = os.path.splitext(prepub_path)[0] + '.markdown'
#       if os.path.isfile(markdown_path):
#         priority = 1
#       elif
  save_file(dest_path, toxml(doc, pretty=True), mtime_mode=MTIME_UPDATE)
